@@ -1,5 +1,5 @@
 /* Darwin support for GDB, the GNU debugger.
-   Copyright (C) 1997-2022 Free Software Foundation, Inc.
+   Copyright (C) 1997-2025 Free Software Foundation, Inc.
 
    Contributed by Apple Computer, Inc.
 
@@ -27,12 +27,11 @@
    the future.  It'd be good to remove this at some point when compiling on
    Tiger is no longer important.  */
 
-#include "defs.h"
 #include "symtab.h"
 #include "gdbtypes.h"
 #include "gdbcore.h"
 #include "value.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
 #include "inferior.h"
 #include "gdbarch.h"
 
@@ -247,7 +246,7 @@ info_mach_ports_command (const char *args, int from_tty)
 	      gdb_printf (_("%u"), ref);
 	    gdb_printf (_(" refs)"));
 	  }
-      
+
       if (task == task_self ())
 	{
 	  if (port == task_self())
@@ -570,9 +569,10 @@ darwin_debug_regions (task_t task, mach_vm_address_t address, int max)
 
       if (print)
 	{
+	  gdbarch *arch = current_inferior ()->arch ();
 	  gdb_printf (_("%s-%s %s/%s  %s %s %s"),
-		      paddress (target_gdbarch (), prev_address),
-		      paddress (target_gdbarch (), prev_address + prev_size),
+		      paddress (arch, prev_address),
+		      paddress (arch, prev_address + prev_size),
 		      unparse_protection (prev_info.protection),
 		      unparse_protection (prev_info.max_protection),
 		      unparse_inheritance (prev_info.inheritance),
@@ -618,7 +618,7 @@ darwin_debug_regions_recurse (task_t task)
 
   ui_out_emit_table table_emitter (uiout, 9, -1, "regions");
 
-  if (gdbarch_addr_bit (target_gdbarch ()) <= 32)
+  if (gdbarch_addr_bit (current_inferior ()->arch ()) <= 32)
     {
       uiout->table_header (10, ui_left, "start", "Start");
       uiout->table_header (10, ui_left, "end", "End");
@@ -630,7 +630,7 @@ darwin_debug_regions_recurse (task_t task)
     }
   uiout->table_header (3, ui_left, "min-prot", "Min");
   uiout->table_header (3, ui_left, "max-prot", "Max");
-  uiout->table_header (5, ui_left, "inheritence", "Inh");
+  uiout->table_header (5, ui_left, "inheritance", "Inh");
   uiout->table_header (9, ui_left, "share-mode", "Shr");
   uiout->table_header (1, ui_left, "depth", "D");
   uiout->table_header (3, ui_left, "submap", "Sm");
@@ -654,14 +654,15 @@ darwin_debug_regions_recurse (task_t task)
 
       {
 	ui_out_emit_tuple tuple_emitter (uiout, "regions-row");
+	gdbarch *arch = current_inferior ()->arch ();
 
-	uiout->field_core_addr ("start", target_gdbarch (), r_start);
-	uiout->field_core_addr ("end", target_gdbarch (), r_start + r_size);
+	uiout->field_core_addr ("start", arch, r_start);
+	uiout->field_core_addr ("end", arch, r_start + r_size);
 	uiout->field_string ("min-prot",
 			     unparse_protection (r_info.protection));
 	uiout->field_string ("max-prot",
 			     unparse_protection (r_info.max_protection));
-	uiout->field_string ("inheritence",
+	uiout->field_string ("inheritance",
 			     unparse_inheritance (r_info.inheritance));
 	uiout->field_string ("share-mode",
 			     unparse_share_mode (r_info.share_mode));
@@ -699,7 +700,7 @@ info_mach_regions_command (const char *args, int from_tty)
   task = get_task_from_args (args);
   if (task == TASK_NULL)
     return;
-  
+
   darwin_debug_regions (task, 0, -1);
 }
 
@@ -711,7 +712,7 @@ info_mach_regions_recurse_command (const char *args, int from_tty)
   task = get_task_from_args (args);
   if (task == TASK_NULL)
     return;
-  
+
   darwin_debug_regions_recurse (task);
 }
 
@@ -723,8 +724,8 @@ info_mach_region_command (const char *exp, int from_tty)
   struct inferior *inf;
 
   expression_up expr = parse_expression (exp);
-  val = evaluate_expression (expr.get ());
-  if (TYPE_IS_REFERENCE (value_type (val)))
+  val = expr->evaluate ();
+  if (TYPE_IS_REFERENCE (val->type ()))
     {
       val = value_ind (val);
     }
@@ -827,7 +828,7 @@ info_mach_exceptions_command (const char *args, int from_tty)
       if (inferior_ptid == null_ptid)
 	gdb_printf (_("No inferior running\n"));
       inf = current_inferior ();
-      
+
       darwin_inferior *priv = get_darwin_inferior (inf);
 
       kret = task_get_exception_ports
@@ -838,9 +839,7 @@ info_mach_exceptions_command (const char *args, int from_tty)
     }
 }
 
-void _initialize_darwin_info_commands ();
-void
-_initialize_darwin_info_commands ()
+INIT_GDB_FILE (darwin_info_commands)
 {
   add_info ("mach-tasks", info_mach_tasks_command,
 	    _("Get list of tasks in system."));
