@@ -46,7 +46,6 @@
 #include "cli/cli-style.h"
 #include "gdbsupport/unordered_map.h"
 
-#include <ctype.h>
 #include <algorithm>
 
 /* This enum represents the values that the user can choose when
@@ -488,7 +487,7 @@ read_mapping (const char *line)
 
   p = skip_spaces (p);
   const char *permissions_start = p;
-  while (*p && !isspace (*p))
+  while (*p && !c_isspace (*p))
     p++;
   mapping.permissions = std::string (permissions_start,
 				     (size_t) (p - permissions_start));
@@ -497,7 +496,7 @@ read_mapping (const char *line)
 
   p = skip_spaces (p);
   const char *device_start = p;
-  while (*p && !isspace (*p))
+  while (*p && !c_isspace (*p))
     p++;
   mapping.device = {device_start, (size_t) (p - device_start)};
 
@@ -652,7 +651,7 @@ mapping_is_anonymous_p (const char *filename)
      be anonymous.  Otherwise, GDB considers this mapping to be a
      file-backed mapping (because there will be a file associated with
      it).
- 
+
      It is worth mentioning that, from all those checks described
      above, the most fragile is the one to see if the file name ends
      with " (deleted)".  This does not necessarily mean that the
@@ -666,7 +665,7 @@ mapping_is_anonymous_p (const char *filename)
      that if the file name ends with " (deleted)", then the mapping is
      indeed anonymous.  FWIW, this is something the Linux kernel could
      do better: expose this information in a more direct way.
- 
+
    - If we see the flag "sh" in the "VmFlags:" field (in
      /proc/PID/smaps), then certainly the memory mapping is shared
      (VM_SHARED).  If we have access to the VmFlags, and we don't see
@@ -843,7 +842,7 @@ linux_info_proc (struct gdbarch *gdbarch, const char *args,
   char filename[100];
   fileio_error target_errno;
 
-  if (args && isdigit (args[0]))
+  if (args && c_isdigit (args[0]))
     {
       char *tem;
 
@@ -1102,12 +1101,12 @@ linux_info_proc (struct gdbarch *gdbarch, const char *args,
 
 /* Implementation of `gdbarch_read_core_file_mappings', as defined in
    gdbarch.h.
-   
+
    This function reads the NT_FILE note (which BFD turns into the
    section ".note.linuxcore.file").  The format of this note / section
    is described as follows in the Linux kernel sources in
    fs/binfmt_elf.c:
-   
+
       long count     -- how many files are mapped
       long page_size -- units for file_ofs
       array of [COUNT] elements of
@@ -1115,14 +1114,14 @@ linux_info_proc (struct gdbarch *gdbarch, const char *args,
 	long end
 	long file_ofs
       followed by COUNT filenames in ASCII: "FILE1" NUL "FILE2" NUL...
-      
+
    CBFD is the BFD of the core file.
 
    PRE_LOOP_CB is the callback function to invoke prior to starting
    the loop which processes individual entries.  This callback will
    only be executed after the note has been examined in enough
    detail to verify that it's not malformed in some way.
-   
+
    LOOP_CB is the callback function that will be executed once
    for each mapping.  */
 
@@ -1307,18 +1306,15 @@ linux_core_info_proc (struct gdbarch *gdbarch, struct bfd *cbfd,
    interface.  */
 
 static LONGEST
-linux_core_xfer_siginfo (struct gdbarch *gdbarch, gdb_byte *readbuf,
-			 ULONGEST offset, ULONGEST len)
+linux_core_xfer_siginfo (struct gdbarch *gdbarch, struct bfd &cbfd,
+			 gdb_byte *readbuf, ULONGEST offset, ULONGEST len)
 {
   thread_section_name section_name (".note.linuxcore.siginfo", inferior_ptid);
-  asection *section
-    = bfd_get_section_by_name (current_program_space->core_bfd (),
-			       section_name.c_str ());
-  if (section == NULL)
+  asection *section = bfd_get_section_by_name (&cbfd, section_name.c_str ());
+  if (section == nullptr)
     return -1;
 
-  if (!bfd_get_section_contents (current_program_space->core_bfd (), section,
-				 readbuf, offset, len))
+  if (!bfd_get_section_contents (&cbfd, section, readbuf, offset, len))
     return -1;
 
   return len;
@@ -1326,8 +1322,8 @@ linux_core_xfer_siginfo (struct gdbarch *gdbarch, gdb_byte *readbuf,
 
 typedef int linux_find_memory_region_ftype (ULONGEST vaddr, ULONGEST size,
 					    ULONGEST offset,
-					    int read, int write,
-					    int exec, int modified,
+					    bool read, bool write,
+					    bool exec, bool modified,
 					    bool memory_tagged,
 					    const std::string &filename,
 					    void *data);
@@ -1515,12 +1511,13 @@ linux_process_address_in_memtag_page (CORE_ADDR address)
 static bool
 linux_core_file_address_in_memtag_page (CORE_ADDR address)
 {
-  if (current_program_space->core_bfd () == nullptr)
+  bfd *cbfd = get_inferior_core_bfd (current_inferior ());
+
+  if (cbfd == nullptr)
     return false;
 
   memtag_section_info info;
-  return get_next_core_memtag_section (current_program_space->core_bfd (),
-				       nullptr, address, info);
+  return get_next_core_memtag_section (cbfd, nullptr, address, info);
 }
 
 /* See linux-tdep.h.  */
@@ -1600,7 +1597,7 @@ linux_find_memory_regions_full (struct gdbarch *gdbarch,
 	{
 	  func (map.start_address, map.end_address - map.start_address,
 		map.offset, map.read, map.write, map.exec,
-		1, /* MODIFIED is true because we want to dump
+		true, /* MODIFIED is true because we want to dump
 		      the mapping.  */
 		map.vmflags.memory_tagging != 0,
 		map.filename, obfd);
@@ -1630,8 +1627,8 @@ struct linux_find_memory_regions_data
 static int
 linux_find_memory_regions_thunk (ULONGEST vaddr, ULONGEST size,
 				 ULONGEST offset,
-				 int read, int write, int exec, int modified,
-				 bool memory_tagged,
+				 bool read, bool write, bool exec,
+				 bool modified, bool memory_tagged,
 				 const std::string &filename, void *arg)
 {
   struct linux_find_memory_regions_data *data
@@ -1687,7 +1684,7 @@ struct linux_make_mappings_data
 static int
 linux_make_mappings_callback (ULONGEST vaddr, ULONGEST size,
 			      ULONGEST offset,
-			      int read, int write, int exec, int modified,
+			      bool read, bool write, bool exec, bool modified,
 			      bool memory_tagged,
 			      const std::string &filename, void *data)
 {
@@ -1740,7 +1737,7 @@ linux_make_mappings_corefile_notes (struct gdbarch *gdbarch, bfd *obfd,
   pack_long (buf, long_type, 1);
   obstack_grow (&data_obstack, buf, long_type->length ());
 
-  linux_find_memory_regions_full (gdbarch, 
+  linux_find_memory_regions_full (gdbarch,
 				  dump_note_entry_p,
 				  linux_make_mappings_callback,
 				  &mapping_data);
@@ -2230,7 +2227,7 @@ linux_fill_prpsinfo (struct elf_internal_linux_prpsinfo *p)
      specifically under the entry of `/proc/[pid]/stat'.  */
 
   /* Getting rid of the PID, since we already have it.  */
-  while (isdigit (*proc_stat))
+  while (c_isdigit (*proc_stat))
     ++proc_stat;
 
   proc_stat = skip_spaces (proc_stat);
@@ -2302,10 +2299,10 @@ linux_fill_prpsinfo (struct elf_internal_linux_prpsinfo *p)
     {
       /* Advancing the pointer to the beginning of the UID.  */
       tmpstr += sizeof ("Uid:");
-      while (*tmpstr != '\0' && !isdigit (*tmpstr))
+      while (*tmpstr != '\0' && !c_isdigit (*tmpstr))
 	++tmpstr;
 
-      if (isdigit (*tmpstr))
+      if (c_isdigit (*tmpstr))
 	p->pr_uid = strtol (tmpstr, &tmpstr, 10);
     }
 
@@ -2315,10 +2312,10 @@ linux_fill_prpsinfo (struct elf_internal_linux_prpsinfo *p)
     {
       /* Advancing the pointer to the beginning of the GID.  */
       tmpstr += sizeof ("Gid:");
-      while (*tmpstr != '\0' && !isdigit (*tmpstr))
+      while (*tmpstr != '\0' && !c_isdigit (*tmpstr))
 	++tmpstr;
 
-      if (isdigit (*tmpstr))
+      if (c_isdigit (*tmpstr))
 	p->pr_gid = strtol (tmpstr, &tmpstr, 10);
     }
 
@@ -2381,9 +2378,9 @@ linux_make_corefile_notes (struct gdbarch *gdbarch, bfd *obfd, int *note_size)
 			     target_thread_architecture (signalled_thr->ptid),
 			     obfd, note_data, note_size, stop_signal);
     }
-  for (thread_info *thr : current_inferior ()->non_exited_threads ())
+  for (thread_info &thr : current_inferior ()->non_exited_threads ())
     {
-      if (thr == signalled_thr)
+      if (&thr == signalled_thr)
 	continue;
 
       /* On some architectures, like AArch64, each thread can have a distinct
@@ -2392,7 +2389,7 @@ linux_make_corefile_notes (struct gdbarch *gdbarch, bfd *obfd, int *note_size)
 
 	 Fetch each thread's gdbarch and pass it down to the lower layers so
 	 we can dump the right set of registers.  */
-      linux_corefile_thread (thr, target_thread_architecture (thr->ptid),
+      linux_corefile_thread (&thr, target_thread_architecture (thr.ptid),
 			     obfd, note_data, note_size, stop_signal);
     }
 
@@ -2697,15 +2694,14 @@ linux_vsyscall_range_raw (struct gdbarch *gdbarch, struct mem_range *range)
       long phdrs_size;
       int num_phdrs, i;
 
-      phdrs_size
-	= bfd_get_elf_phdr_upper_bound (current_program_space->core_bfd ());
+      bfd *cbfd = get_inferior_core_bfd (current_inferior ());
+      phdrs_size = bfd_get_elf_phdr_upper_bound (cbfd);
       if (phdrs_size == -1)
 	return 0;
 
       gdb::unique_xmalloc_ptr<Elf_Internal_Phdr>
 	phdrs ((Elf_Internal_Phdr *) xmalloc (phdrs_size));
-      num_phdrs = bfd_get_elf_phdrs (current_program_space->core_bfd (),
-				     phdrs.get ());
+      num_phdrs = bfd_get_elf_phdrs (cbfd, phdrs.get ());
       if (num_phdrs == -1)
 	return 0;
 
